# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-19 21:33
from __future__ import unicode_literals

from django.db import migrations
import djcelery.picklefield


class Migration(migrations.Migration):

    dependencies = [
        ('djcelery', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='crontabschedule',
            name='nowfun',
            field=djcelery.picklefield.PickledObjectField(default=None, editable=False),
        ),
    ]
